/*
** Copyright (c) 1995-2002
**      Daniel O'Connor and Genesis Software.  All rights reserved.
**
** Originally written by Michael Smith.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer as
**    the first lines of this file unmodified.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. The name of the author may not be used to endorse or promote products
**    derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY Daniel O'Connor AND Genesis Software ``AS IS'' 
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED. IN NO EVENT SHALL Michael Smith AND OR Genesis Software BE
** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
** POSSIBILITY OF SUCH DAMAGE.
**
**      $Id: mdsioreg.h,v 1.3 2002/03/25 02:34:42 doconnor Exp $
*/

/*
** Register defines for the MDSIO card
*/

/*
 * Configuration space registers in the PLX 9050 
 */
#define PLX_PCIBAR0	0x10		/* PCI base address register for memory access to local cfg regs */
#define PLX_PCIBAR1	0x14		/* PCI base address register for I/O access to local cfg regs */
#define	PLX_PCIBAR2	0x18		/* PCI base address register for memory access to local addr space 0 */
#define PLX_PCIBAR3	0x1c		/* PCI base address register for memory access to local addr space 1 */
#define PLX_PCIBAR4	0x20		/* PCI base address register for memory access to local addr space 2 */
#define PLX_PCIBAR5	0x24		/* PCI base address register for memory access to local addr space 3 */

/*
 * PLX Local Control Registers
 */
#define PLX_LAS0RR	0x00
#define PLX_LAS1RR	0x04
#define PLX_LAS2RR	0x08
#define PLX_LAS3RR	0x0c
#define	PLX_EROMRR	0x10
#define	PLX_LAS0BA	0x14
#define	PLX_LAS1BA	0x18
#define	PLX_LAS2BA	0x1c
#define	PLX_LAS3BA	0x20
#define	PLX_EROMBA	0x24
#define	PLX_LAS0BRD	0x28
#define	PLX_LAS1BRD	0x2c
#define	PLX_LAS2BRD	0x30
#define	PLX_LAS3BRD	0x34
#define	PLX_EROMBRD	0x38
#define	PLX_CS0BASE	0x3c
#define	PLX_CS1BASE	0x40
#define	PLX_CS2BASE	0x44
#define	PLX_CS3BASE	0x48
#define	PLX_INTCSR	0x4c
#define	PLX_CNTRL	0x50

/*
 * Interrupt related items
 */
#define PLX_INTR1_ENABLE	(1<<0)
#define PLX_INTR1_POL		(1<<1)
#define PLX_INTR1_STATUS	(1<<2)
#define PLX_INTR1_MASK		(PLX_INTR1_ENABLE | PLX_INTR1_STATUS)
#define PLX_INTR2_ENABLE	(1<<3)
#define PLX_INTR2_POL		(1<<4)
#define PLX_INTR2_STATUS	(1<<5)
#define PLX_INTR2_MASK		(PLX_INTR2_ENABLE | PLX_INTR2_STATUS)
#define PLX_INTRP_ENABLE	(1<<6)

#define m_inb(s, p)							\
        bus_space_read_1((s).sc_st, (s).sc_sh, (p))

#define m_inl(s, p)							\
        bus_space_read_4((s).sc_st, (s).sc_sh, (p))

#define m_outb(s, p, v)							\
        bus_space_write_1((s).sc_st, (s).sc_sh, (p), (v)); DELAY(1000)

#define m_outl(s, p, v)							\
        bus_space_write_4((s).sc_st, (s).sc_sh, (p), (v)); DELAY(1000)

