/*
** Copyright (c) 1995-2002
**      Daniel O'Connor and Genesis Software.  All rights reserved.
**
** Originally written by Michael Smith.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer as
**    the first lines of this file unmodified.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. The name of the author may not be used to endorse or promote products
**    derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY Michael Smith AND Genesis Software ``AS IS'' 
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED. IN NO EVENT SHALL Michael Smith AND OR Genesis Software BE
** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
** POSSIBILITY OF SUCH DAMAGE.
**
**      $Id: mdsiovar.h,v 1.3 2002/03/25 02:34:42 doconnor Exp $
*/

/*******************************************************************************
** Handle parameter overrides from the kernel config file
*/
#ifndef MDSIO_RXBUF
#define MDSIO_RXBUF	1024
#else
#if MDSIO_RXBUF > 65536
#error MDSIO_RXBUF too large
#endif
#endif

#ifndef MDSIO_TXBUF
#define MDSIO_TXBUF	1024
#else
#if MDSIO_TXBUF > 65536
#error MDSIO_TXBUF too large
#endif
#endif

/* Timeout specified as 1/MDSIO_TIMEOUT seconds */
#ifndef MDSIO_TIMEOUT
#define MDSIO_TIMEOUT	64
#endif

#ifndef MDSIO_FTL
#define MDSIO_FTL	FIFO_RX_MEDH
#endif

#ifndef __i386__
#define disable_intr()   
#define enable_intr()   
#endif
 
#ifdef SMP
#define disable_intr()  COM_DISABLE_INTR()
#define enable_intr()   COM_ENABLE_INTR()
#endif /* SMP */

/*******************************************************************************
** Per-port structure and related definitions
*/
/*
 *Information newbus needs to do bus_space things
 */
struct __md_res {
    struct resource	*reshandle;	/* Resource handle */
    bus_space_tag_t	sc_st;		/* bus space tag */
    bus_space_handle_t	sc_sh;		/* bus space handle */
    int			rid;		/* Resource ID */
};

struct mdsio_softc
{
    u_short		state;			/* current port state */
#define MD_OPEN		(1<<0)			/* port is open */
#define MD_READSLEEP	(1<<1)			/* sleeping for read */
#define MD_XMITTING	(1<<2)			/* transmitting */
#define MD_PREADDR	(1<<3)			/* waiting for SR to drain before sending address */
#define MD_POSTADDR	(1<<4)			/* waiting for address to be sent before going to data */
#define MD_WRITESLEEP	(1<<5)
#define MD_TIMER	(1<<6)			/* there's a timeout routine active on this interface */
#define MD_TAILDRAIN	(1<<7)			/* waiting for the last byte to drain before disabling Tx */
#define MD_DRAINSLEEP	(1<<8)			/* sleeping for drain so we can close */

    struct selinfo	rsel,wsel;		/* Information for select() */

    struct __md_res	iobase;			/* base port resource handle */

    struct resource	*irqres;		/* Newbus interrupt tag */
    void		*irq_ih;		/* IRQ Handle */
    int			irqrid;

    dev_t		dev_handle;		/* Device handle so we can unregister on detach */

    struct devstat	*md_ds;			/* Device stats handle */

    struct callout_handle tmhn;			/* Timer handle */
    struct callout_handle comphn;		/* Whiner timer handle */
    
    u_char		hasfifo;
    int			unit;			/* unit number */
    int			eflag;
#define MD_FIFOOF	(1<<0)			/* FIFO overflow */
#define MD_IBUFOF	(1<<1)			/* ringbuffer overflow */
#define MD_FERROR	(1<<2)			/* framing error (noise) */
#define MD_RXWHILETX	(1<<3)			/* received data while transmitting */

    int			speed;
    int			options;		/* port configuration options */
#define MD_TXCTL	(1<<0)			/* Tx/Rx control required */
#define MD_2WIRE	(1<<1)			/* 2-wire mode */

    u_char		tx_mcr;			/* MCR values for each state */
    u_char		rx_mcr;
    u_char		cur_mcr;		/* current MCR value */

    u_short		rxbuf[MDSIO_RXBUF];	/* receiver ringbuffer */
    u_short		rxtail,rxhead;

    u_short		txbuf[MDSIO_TXBUF];	/* transmitter ringbuffer */
    u_short		txtail,txhead;

    u_long		ints;			/* total interrupts */
    u_long		rxbytes;		/* bytes received */
    u_long		txbytes;		/* bytes sent */
    u_long		errors;			/* total errors */

    u_long		clk;			/* Baud clock (before division) */    

    struct sysctl_ctx_list sysctl_ctx;		/* Sysctl magic */
    struct sysctl_oid 	*sysctl_tree;
};

/*******************************************************************************
** Flag decoding macros
*/
#define RTS_TXCONTROL(dev)	(device_get_flags(dev) & 0x04)
#define DTR_TXCONTROL(dev)	(device_get_flags(dev) & 0x08)
#define TXCTRL_INVERT(dev)	(device_get_flags(dev) & 0x10)
#define MDSIO_TXCONTROL(dev)	(device_get_flags(dev) & 0x0c)
#define MDSIO_2WIRE(dev)	(device_get_flags(dev) & 0x20)

#define MD_ADDRMARK		0x100
#define LCR_NORM		(CFCR_8BITS|CFCR_PENAB|CFCR_PEVEN|CFCR_PONE)	/* normal (listen) mode */
#define LCR_ADDR		(CFCR_8BITS|CFCR_PENAB|CFCR_PODD|CFCR_PONE)	/* addressmark mode */

#define com_scr			7	/* scratch register for 16450-16550 (R/W) */

/*******************************************************************************
** Buffer manipulation macros
*/
#define rxbuflen(x)		((x->rxhead + MDSIO_RXBUF - x->rxtail) % MDSIO_RXBUF)
#define rxbuffree(x)		(MDSIO_RXBUF - rxbuflen(x))
#define rxheadnext(x)		((x->rxhead+1)%MDSIO_RXBUF)
#define txbuflen(x)		((x->txhead + MDSIO_TXBUF - x->txtail) % MDSIO_TXBUF)
#define txbuffree(x)		(MDSIO_TXBUF - txbuflen(x))
#define txtailnext(x)		((x->txtail+1)%MDSIO_TXBUF)


/*******************************************************************************
** Misc defines/macros
*/
/* Set the mcr for Tx (m==MD_TX) or Rx (m==MD_RX) */
#define set_mcr(x,m)		m_outb((x)->iobase, com_mcr, (x)->cur_mcr = (m ? (x)->tx_mcr:(x)->rx_mcr))
#define MD_TX			1
#define MD_RX			0

/*******************************************************************************
** Shared administrivia
*/

#define CDEV_MAJOR	20
#define DEVNAME		"mdsio"

static devclass_t mdsio_devclass;
/*******************************************************************************
** PCI administrivia
*/

#ifdef NOTYET
#define PCI_VND_ID 0x4753 /* 'GS' */
#define PCI_DEV_ID 0x8001
#else
#define PCI_VND_ID 0x6666 /* Decision */
#define PCI_DEV_ID 0x0004 /* 2 Port serial */
#endif

